"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const simulator_xcode_8_1 = __importDefault(require("./simulator-xcode-8"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const support_1 = require("@appium/support");
const async_lock_1 = __importDefault(require("async-lock"));
const logger_1 = __importDefault(require("./logger"));
const asyncbox_1 = require("asyncbox");
const utils_js_1 = require("./utils.js");
const defaults_utils_1 = require("./defaults-utils");
const bluebird_1 = __importDefault(require("bluebird"));
const events_1 = require("events");
const appium_xcode_1 = require("appium-xcode");
const teen_process_1 = require("teen_process");
const SIMULATOR_SHUTDOWN_TIMEOUT = 15 * 1000;
const startupLock = new async_lock_1.default();
const preferencesPlistGuard = new async_lock_1.default();
const ENROLLMENT_NOTIFICATION_RECEIVER = 'com.apple.BiometricKit.enrollmentChanged';
const DOMAIN_KEYBOARD_PREFERENCES = 'com.apple.keyboard.preferences';
// com.apple.SpringBoard: translates com.apple.SpringBoard and system prompts for push notification
// com.apple.locationd: translates system prompts for location
// com.apple.tccd: translates system prompts for camera, microphone, contact, photos and app tracking transparency
const SERVICES_FOR_TRANSLATION = ['com.apple.SpringBoard', 'com.apple.locationd', 'com.apple.tccd'];
const GLOBAL_PREFS_PLIST = '.GlobalPreferences.plist';
/**
 * Creates device and common Simulator preferences, which could
 * be later applied using `defaults` CLI utility.
 *
 * @param {Partial<RunOptions>} opts
 * @returns {any[]} The first array item is the resulting device preferences
 * object and the second one is common preferences object
 */
function compileSimulatorPreferences(opts = {}) {
    const { connectHardwareKeyboard, tracePointer, pasteboardAutomaticSync, scaleFactor, } = opts;
    const commonPreferences = {
        // This option is necessary to make the Simulator window follow
        // the actual XCUIDevice orientation
        RotateWindowWhenSignaledByGuest: true,
        // https://github.com/appium/appium/issues/16418
        StartLastDeviceOnLaunch: false,
        DetachOnWindowClose: false,
        AttachBootedOnStart: true,
    };
    const devicePreferences = opts.devicePreferences ? lodash_1.default.cloneDeep(opts.devicePreferences) : {};
    if (scaleFactor) {
        devicePreferences.SimulatorWindowLastScale = parseFloat(scaleFactor);
    }
    if (lodash_1.default.isBoolean(connectHardwareKeyboard) || lodash_1.default.isNil(connectHardwareKeyboard)) {
        devicePreferences.ConnectHardwareKeyboard = connectHardwareKeyboard ?? false;
        commonPreferences.ConnectHardwareKeyboard = connectHardwareKeyboard ?? false;
    }
    if (lodash_1.default.isBoolean(tracePointer)) {
        commonPreferences.ShowSingleTouches = tracePointer;
        commonPreferences.ShowPinches = tracePointer;
        commonPreferences.ShowPinchPivotPoint = tracePointer;
        commonPreferences.HighlightEdgeGestures = tracePointer;
    }
    switch (lodash_1.default.lowerCase(pasteboardAutomaticSync)) {
        case 'on':
            commonPreferences.PasteboardAutomaticSync = true;
            break;
        case 'off':
            // Improve launching simulator performance
            // https://github.com/WebKit/webkit/blob/master/Tools/Scripts/webkitpy/xcode/simulated_device.py#L413
            commonPreferences.PasteboardAutomaticSync = false;
            break;
        case 'system':
            // Do not add -PasteboardAutomaticSync
            break;
        default:
            logger_1.default.info(`['on', 'off' or 'system'] are available as the pasteboard automatic sync option. Defaulting to 'off'`);
            commonPreferences.PasteboardAutomaticSync = false;
    }
    return [devicePreferences, commonPreferences];
}
class SimulatorXcode9 extends simulator_xcode_8_1.default {
    constructor(udid, xcodeVersion) {
        super(udid, xcodeVersion);
    }
    /**
     * @typedef {Object} DevicePreferences
     * @property {?number} SimulatorExternalDisplay - TBD. Example value: 2.114
     * @property {?string} ChromeTint - TBD. Example value: ''
     * @property {?number} SimulatorWindowLastScale - Scale value for the particular Simulator window.
     *                                                1.0 means 100% scale.
     * @property {?string} SimulatorWindowOrientation - Simulator window orientation. Possible values are:
     *                                                  'Portrait', 'LandscapeLeft', 'PortraitUpsideDown' and 'LandscapeRight'.
     * @property {?number} SimulatorWindowRotationAngle - Window rotation angle. This value is expected to be in sync
     *                                                    with _SimulatorWindowOrientation_. The corresponding values are:
     *                                                    0, 90, 180 and 270.
     * @property {?string} SimulatorWindowCenter - The coordinates of Simulator's window center in pixels,
     *                                             for example '{-1294.5, 775.5}'.
     * @property {?boolean} ConnectHardwareKeyboard - Equals to 1 if hardware keyboard should be connected.
     *                                                Otherwise 0.
     */
    /**
     * @typedef {Object} CommonPreferences
     * @property {boolean} ConnectHardwareKeyboard - Whether to connect hardware keyboard
     */
    /**
     * @typedef {Object} RunOptions
     * @property {string} scaleFactor: Any positive float value. 1.0 means 1:1 scale.
     * Defines the window scale value for the UI client window for the current Simulator.
     * Equals to `null` by default, which keeps the current scale unchanged.
     * @property {boolean} connectHardwareKeyboard: whether to connect the hardware keyboard to the
     * Simulator UI client. Equals to `false` by default.
     * @property {number} startupTimeout: number of milliseconds to wait until Simulator booting
     * process is completed. The default timeout will be used if not set explicitly.
     * @property {boolean} isHeadless: whether to start the Simulator in headless mode (with UI
     * client invisible). `false` by default.
     * @property {?boolean} tracePointer [false] - Whether to highlight touches on Simulator
     * screen. This is helpful while debugging automated tests or while observing the automation
     * recordings.
     * @property {string} pasteboardAutomaticSync ['off'] - Whether to disable pasteboard sync with the
     * Simulator UI client or respect the system wide preference. 'on', 'off', or 'system' is available.
     * The sync increases launching simulator process time, but it allows system to sync pasteboard
     * with simulators. Follows system-wide preference if the value is 'system'.
     * Defaults to 'off'.
     * @property {DevicePreferences} devicePreferences: preferences of the newly created Simulator
     * device
     */
    /**
     * Executes given Simulator with options. The Simulator will not be restarted if
     * it is already running and the current UI state matches to `isHeadless` option.
     * @override
     *
     * @param {Partial<RunOptions>} opts - One or more of available Simulator options
     */
    async run(opts = {}) {
        opts = lodash_1.default.cloneDeep(opts);
        lodash_1.default.defaultsDeep(opts, {
            isHeadless: false,
            startupTimeout: this.startupTimeout,
        });
        const [devicePreferences, commonPreferences] = compileSimulatorPreferences(opts);
        await this.updatePreferences(devicePreferences, commonPreferences);
        const timer = new support_1.timing.Timer().start();
        const shouldWaitForBoot = await startupLock.acquire(this.uiClientBundleId, async () => {
            const isServerRunning = await this.isRunning();
            const uiClientPid = await this.getUIClientPid();
            if (opts.isHeadless) {
                if (isServerRunning && !uiClientPid) {
                    logger_1.default.info(`Simulator with UDID '${this.udid}' is already booted in headless mode.`);
                    return false;
                }
                if (await this.killUIClient({ pid: uiClientPid })) {
                    logger_1.default.info(`Detected the Simulator UI client was running and killed it. Verifying the current Simulator state`);
                }
                try {
                    // Stopping the UI client kills all running servers for some early XCode versions. This is a known bug
                    await (0, asyncbox_1.waitForCondition)(async () => await this.isShutdown(), {
                        waitMs: 5000,
                        intervalMs: 100,
                    });
                }
                catch (e) {
                    if (!await this.isRunning()) {
                        throw new Error(`Simulator with UDID '${this.udid}' cannot be transitioned to headless mode`);
                    }
                    return false;
                }
                logger_1.default.info(`Booting Simulator with UDID '${this.udid}' in headless mode. ` +
                    `All UI-related capabilities are going to be ignored`);
                await this.boot();
            }
            else {
                if (isServerRunning && uiClientPid) {
                    logger_1.default.info(`Both Simulator with UDID '${this.udid}' and the UI client are currently running`);
                    return false;
                }
                if (isServerRunning) {
                    logger_1.default.info(`Simulator '${this.udid}' is booted while its UI is not visible. ` +
                        `Trying to restart it with the Simulator window visible`);
                    await this.shutdown({ timeout: SIMULATOR_SHUTDOWN_TIMEOUT });
                }
                await this.launchWindow(Boolean(uiClientPid), opts);
            }
            return true;
        });
        if (shouldWaitForBoot && opts.startupTimeout) {
            await this.waitForBoot(opts.startupTimeout);
            logger_1.default.info(`Simulator with UDID ${this.udid} booted in ${timer.getDuration().asSeconds.toFixed(3)}s`);
        }
        (async () => {
            try {
                await this.disableKeyboardIntroduction();
            }
            catch (e) {
                logger_1.default.info(`Cannot disable Simulator keyboard introduction. Original error: ${e.message}`);
            }
        })();
    }
    /**
     * @override
     */
    async startUIClient(opts = {}) {
        opts = lodash_1.default.cloneDeep(opts);
        lodash_1.default.defaultsDeep(opts, {
            startupTimeout: this.startupTimeout,
        });
        const simulatorApp = path_1.default.resolve(await (0, appium_xcode_1.getPath)(), 'Applications', utils_js_1.SIMULATOR_APP_NAME);
        const args = ['-Fn', simulatorApp];
        logger_1.default.info(`Starting Simulator UI: ${support_1.util.quote(['open', ...args])}`);
        try {
            await (0, teen_process_1.exec)('open', args, { timeout: opts.startupTimeout });
        }
        catch (err) {
            throw new Error(`Got an unexpected error while opening Simulator UI: ` +
                err.stderr || err.stdout || err.message);
        }
    }
    /**
     * Disable keyboard tutorial as 'com.apple.keyboard.preferences' domain via 'defaults' command.
     */
    async disableKeyboardIntroduction() {
        const argChunks = (0, defaults_utils_1.generateDefaultsCommandArgs)({
            // To disable 'DidShowContinuousPathIntroduction' for iOS 15+ simulators since changing the preference via WDA
            // does not work on them. Lower than the versions also can have this preference, but nothing happen.
            DidShowContinuousPathIntroduction: 1
        }, true);
        await bluebird_1.default.all(argChunks.map((args) => this.simctl.spawnProcess([
            'defaults', 'write', DOMAIN_KEYBOARD_PREFERENCES, ...args
        ])));
    }
    /***
     * Boots simulator and opens simulators UI Client if not already opened.
     *
     * @param {boolean} isUiClientRunning - process id of simulator UI client.
     * @param {Partial<RunOptions>} opts - arguments to start simulator UI client with.
     */
    async launchWindow(isUiClientRunning, opts = {}) {
        await this.boot();
        if (!isUiClientRunning) {
            await this.startUIClient(opts);
        }
    }
    /**
     * Boots Simulator if not already booted.
     * Does nothing if it is already running.
     * This API does NOT wait until Simulator is fully booted.
     *
     * @throws {Error} If there was a failure while booting the Simulator.
     */
    async boot() {
        const bootEventsEmitter = new events_1.EventEmitter();
        await this.simctl.startBootMonitor({
            onError: (err) => bootEventsEmitter.emit('failure', err),
            onFinished: () => bootEventsEmitter.emit('finish'),
            shouldPreboot: true,
        });
        try {
            await new bluebird_1.default((resolve, reject) => {
                // Historically this call was always asynchronous,
                // e.g. it was not waiting until Simulator is fully booted.
                // So we preserve that behavior, and if no errors are received for a while
                // then we assume the Simulator booting is still in progress.
                setTimeout(resolve, 3000);
                bootEventsEmitter.once('failure', (err) => {
                    if (lodash_1.default.includes(err?.message, 'state: Booted')) {
                        resolve();
                    }
                    else {
                        reject(err);
                    }
                });
                bootEventsEmitter.once('finish', resolve);
            });
        }
        finally {
            bootEventsEmitter.removeAllListeners();
        }
    }
    /**
     * Perform verification of device preferences correctness.
     *
     * @param {Partial<DevicePreferences>} prefs [{}] - The preferences to be verified
     * @throws {Error} If any of the given preference values does not match the expected
     * format.
     */
    verifyDevicePreferences(prefs = {}) {
        if (lodash_1.default.isEmpty(prefs)) {
            return;
        }
        if (!lodash_1.default.isUndefined(prefs.SimulatorWindowLastScale)) {
            if (!lodash_1.default.isNumber(prefs.SimulatorWindowLastScale) || prefs.SimulatorWindowLastScale <= 0) {
                logger_1.default.errorAndThrow(`SimulatorWindowLastScale is expected to be a positive float value. ` +
                    `'${prefs.SimulatorWindowLastScale}' is assigned instead.`);
            }
        }
        if (!lodash_1.default.isUndefined(prefs.SimulatorWindowCenter)) {
            // https://regex101.com/r/2ZXOij/2
            const verificationPattern = /{-?\d+(\.\d+)?,-?\d+(\.\d+)?}/;
            if (!lodash_1.default.isString(prefs.SimulatorWindowCenter) || !verificationPattern.test(prefs.SimulatorWindowCenter)) {
                logger_1.default.errorAndThrow(`SimulatorWindowCenter is expected to match "{floatXPosition,floatYPosition}" format (without spaces). ` +
                    `'${prefs.SimulatorWindowCenter}' is assigned instead.`);
            }
        }
        if (!lodash_1.default.isUndefined(prefs.SimulatorWindowOrientation)) {
            const acceptableValues = ['Portrait', 'LandscapeLeft', 'PortraitUpsideDown', 'LandscapeRight'];
            if (!prefs.SimulatorWindowOrientation || !acceptableValues.includes(prefs.SimulatorWindowOrientation)) {
                logger_1.default.errorAndThrow(`SimulatorWindowOrientation is expected to be one of ${acceptableValues}. ` +
                    `'${prefs.SimulatorWindowOrientation}' is assigned instead.`);
            }
        }
        if (!lodash_1.default.isUndefined(prefs.SimulatorWindowRotationAngle)) {
            if (!lodash_1.default.isNumber(prefs.SimulatorWindowRotationAngle)) {
                logger_1.default.errorAndThrow(`SimulatorWindowRotationAngle is expected to be a valid number. ` +
                    `'${prefs.SimulatorWindowRotationAngle}' is assigned instead.`);
            }
        }
    }
    /**
     * Update the common iOS Simulator preferences file with new values.
     * It is necessary to restart the corresponding Simulator before
     * these changes are applied.
     *
     * @param {Partial<DevicePreferences>} devicePrefs [{}] - The mapping, which represents new device preference values
     * for the given Simulator.
     * @param {Partial<CommonPreferences>} commonPrefs [{}] - The mapping, which represents new common preference values
     * for all Simulators.
     * @return {Promise<boolean>} True if the preferences were successfully updated.
     */
    async updatePreferences(devicePrefs = {}, commonPrefs = {}) {
        if (!lodash_1.default.isEmpty(devicePrefs)) {
            logger_1.default.debug(`Setting preferences of ${this.udid} Simulator to ${JSON.stringify(devicePrefs)}`);
        }
        if (!lodash_1.default.isEmpty(commonPrefs)) {
            logger_1.default.debug(`Setting common Simulator preferences to ${JSON.stringify(commonPrefs)}`);
        }
        const homeFolderPath = process.env.HOME;
        if (!homeFolderPath) {
            logger_1.default.warn(`Cannot get the path to HOME folder from the process environment. ` +
                `Ignoring Simulator preferences update.`);
            return false;
        }
        this.verifyDevicePreferences(devicePrefs);
        const plistPath = path_1.default.resolve(homeFolderPath, 'Library', 'Preferences', 'com.apple.iphonesimulator.plist');
        return await preferencesPlistGuard.acquire(SimulatorXcode9.name, async () => {
            const defaults = new defaults_utils_1.NSUserDefaults(plistPath);
            const prefsToUpdate = lodash_1.default.clone(commonPrefs);
            try {
                if (!lodash_1.default.isEmpty(devicePrefs)) {
                    let existingDevicePrefs;
                    const udidKey = this.udid.toUpperCase();
                    if (await support_1.fs.exists(plistPath)) {
                        const currentPlistContent = await defaults.asJson();
                        if (lodash_1.default.isPlainObject(currentPlistContent.DevicePreferences)
                            && lodash_1.default.isPlainObject(currentPlistContent.DevicePreferences[udidKey])) {
                            existingDevicePrefs = currentPlistContent.DevicePreferences[udidKey];
                        }
                    }
                    Object.assign(prefsToUpdate, {
                        DevicePreferences: {
                            [udidKey]: Object.assign({}, existingDevicePrefs || {}, devicePrefs)
                        }
                    });
                }
                await defaults.update(prefsToUpdate);
                logger_1.default.debug(`Updated ${this.udid} Simulator preferences at '${plistPath}' with ` +
                    JSON.stringify(prefsToUpdate));
                return true;
            }
            catch (e) {
                logger_1.default.warn(`Cannot update ${this.udid} Simulator preferences at '${plistPath}'. ` +
                    `Try to delete the file manually in order to reset it. Original error: ${e.message}`);
                return false;
            }
        });
    }
    /**
     * @inheritdoc
     * @override
     * @protected
     */
    async _activateWindow() {
        let selfName;
        let selfSdk;
        let bootedDevicesCount = 0;
        for (const [sdk, deviceArr] of lodash_1.default.toPairs(await this.simctl.getDevices())) {
            for (const { state, udid, name } of deviceArr) {
                if (state === 'Booted') {
                    bootedDevicesCount++;
                }
                if (!selfName && udid === this.udid) {
                    selfSdk = sdk;
                    selfName = name;
                }
            }
        }
        if (bootedDevicesCount < 2) {
            return await super._activateWindow();
        }
        // There are potentially more that one Simulator window
        return `
      tell application "System Events"
        tell process "Simulator"
          set frontmost to false
          set frontmost to true
          click (menu item 1 where (its name contains "${selfName} " and its name contains "${selfSdk}")) of menu 1 of menu bar item "Window" of menu bar 1
        end tell
      end tell
    `;
    }
    /**
     * @returns {Promise<boolean>}
     */
    async isBiometricEnrolled() {
        const { stdout } = await this.simctl.spawnProcess([
            'notifyutil',
            '-g', ENROLLMENT_NOTIFICATION_RECEIVER
        ]);
        const match = (new RegExp(`${lodash_1.default.escapeRegExp(ENROLLMENT_NOTIFICATION_RECEIVER)}\\s+([01])`))
            .exec(stdout);
        if (!match) {
            throw new Error(`Cannot parse biometric enrollment state from '${stdout}'`);
        }
        logger_1.default.info(`Current biometric enrolled state for ${this.udid} Simulator: ${match[1]}`);
        return match[1] === '1';
    }
    /**
     *
     * @param {boolean} isEnabled
     */
    async enrollBiometric(isEnabled = true) {
        logger_1.default.debug(`Setting biometric enrolled state for ${this.udid} Simulator to '${isEnabled ? 'enabled' : 'disabled'}'`);
        await this.simctl.spawnProcess([
            'notifyutil',
            '-s', ENROLLMENT_NOTIFICATION_RECEIVER, isEnabled ? '1' : '0'
        ]);
        await this.simctl.spawnProcess([
            'notifyutil',
            '-p', ENROLLMENT_NOTIFICATION_RECEIVER
        ]);
        if (await this.isBiometricEnrolled() !== isEnabled) {
            throw new Error(`Cannot set biometric enrolled state for ${this.udid} Simulator to '${isEnabled ? 'enabled' : 'disabled'}'`);
        }
    }
    /**
     * Sends a notification to match/not match the particular biometric.
     *
     * @param {boolean} shouldMatch [true] - Set it to true or false in order to emulate
     * matching/not matching the corresponding biometric
     * @param {string} biometricName [touchId] - Either touchId or faceId (faceId is only available since iOS 11)
     */
    async sendBiometricMatch(shouldMatch = true, biometricName = 'touchId') {
        const domainComponent = (0, utils_js_1.toBiometricDomainComponent)(biometricName);
        const domain = `com.apple.BiometricKit_Sim.${domainComponent}.${shouldMatch ? '' : 'no'}match`;
        await this.simctl.spawnProcess([
            'notifyutil',
            '-p', domain
        ]);
        logger_1.default.info(`Sent notification ${domain} to ${shouldMatch ? 'match' : 'not match'} ${biometricName} biometric ` +
            `for ${this.udid} Simulator`);
    }
    /**
     * @returns {Promise<string>}
     */
    async getLaunchDaemonsRoot() {
        const devRoot = await (0, utils_js_1.getDeveloperRoot)();
        return path_1.default.resolve(devRoot, 'Platforms/iPhoneOS.platform/Developer/Library/CoreSimulator/Profiles/Runtimes/iOS.simruntime/Contents/Resources/RuntimeRoot/System/Library/LaunchDaemons');
    }
    /**
     * @typedef {Object} KeyboardOptions
     * @property {string} name The name of the keyboard locale, for example `en_US` or `de_CH`
     * @property {string} layout The keyboard layout, for example `QUERTY` or `Ukrainian`
     * @property {string?} hardware Could either be `Automatic` or `null`
     */
    /**
     * @typedef {Object} LanguageOptions
     * @property {string} name The name of the language, for example `de` or `zh-Hant-CN`
     * @property {boolean} [skipSyncUiDialogTranslation] no Simulator services will be reset if this option is set to true.
     *                                                   See https://github.com/appium/appium/issues/19440 for more details
     */
    /**
     * @typedef {Object} LocaleOptions
     * @property {string} name The name of the system locale, for example `de_CH` or `zh_CN`
     * @property {string} calendar Optional calendar format, for example `gregorian` or `persian`
     */
    /**
     * @typedef {Object} LocalizationOptions
     * @property {KeyboardOptions} keyboard
     * @property {LanguageOptions} language
     * @property {LocaleOptions} locale
     */
    /**
     * Change localization settings on the currently booted simulator
     *
     * @param {Partial<LocalizationOptions>} opts
     * @throws {Error} If there was a failure while setting the preferences
     * @returns {Promise<boolean>} `true` if any of settings has been successfully changed
     */
    async configureLocalization(opts = {}) {
        if (lodash_1.default.isEmpty(opts)) {
            return false;
        }
        const { language, locale, keyboard } = opts;
        const globalPrefs = {};
        let keyboardId = null;
        if (lodash_1.default.isPlainObject(keyboard)) {
            // @ts-ignore The above check ensures keyboard is what it should be
            const { name, layout, hardware } = keyboard;
            if (!name) {
                throw new Error(`The 'keyboard' field must have a valid name set`);
            }
            if (!layout) {
                throw new Error(`The 'keyboard' field must have a valid layout set`);
            }
            keyboardId = `${name}@sw=${layout}`;
            if (hardware) {
                keyboardId += `;@hw=${hardware}`;
            }
            globalPrefs.AppleKeyboards = [keyboardId];
        }
        if (lodash_1.default.isPlainObject(language)) {
            // @ts-ignore The above check ensures language is what it should be
            const { name } = language;
            if (!name) {
                throw new Error(`The 'language' field must have a valid name set`);
            }
            globalPrefs.AppleLanguages = [name];
        }
        if (lodash_1.default.isPlainObject(locale)) {
            // @ts-ignore The above check ensures locale is what it should be
            const { name, calendar } = locale;
            if (!name) {
                throw new Error(`The 'locale' field must have a valid name set`);
            }
            let localeId = name;
            if (calendar) {
                localeId += `@calendar=${calendar}`;
            }
            globalPrefs.AppleLocale = localeId;
        }
        if (lodash_1.default.isEmpty(globalPrefs)) {
            return false;
        }
        let previousAppleLanguages = null;
        if (globalPrefs.AppleLanguages) {
            const absolutePrefsPath = path_1.default.join(this.getDir(), 'Library', 'Preferences', GLOBAL_PREFS_PLIST);
            try {
                const { stdout } = await (0, teen_process_1.exec)('plutil', ['-convert', 'json', absolutePrefsPath, '-o', '-']);
                previousAppleLanguages = JSON.parse(stdout).AppleLanguages;
            }
            catch (e) {
                logger_1.default.debug(`Cannot retrieve the current value of the 'AppleLanguages' preference: ${e.message}`);
            }
        }
        const argChunks = (0, defaults_utils_1.generateDefaultsCommandArgs)(globalPrefs, true);
        await bluebird_1.default.all(argChunks.map((args) => this.simctl.spawnProcess([
            'defaults', 'write', GLOBAL_PREFS_PLIST, ...args
        ])));
        if (keyboard && keyboardId) {
            const argChunks = (0, defaults_utils_1.generateDefaultsCommandArgs)({
                KeyboardsCurrentAndNext: [keyboardId],
                KeyboardLastUsed: keyboardId,
                KeyboardLastUsedForLanguage: { [keyboard.name]: keyboardId }
            }, true);
            await bluebird_1.default.all(argChunks.map((args) => this.simctl.spawnProcess([
                'defaults', 'write', 'com.apple.Preferences', ...args
            ])));
        }
        if (globalPrefs.AppleLanguages) {
            if (lodash_1.default.isEqual(previousAppleLanguages, globalPrefs.AppleLanguages)) {
                logger_1.default.info(`The 'AppleLanguages' preference is already set to '${globalPrefs.AppleLanguages}'. ` +
                    `Skipping services reset`);
            }
            else if (language?.skipSyncUiDialogTranslation) {
                logger_1.default.info('Skipping services reset as requested. This might leave some system UI alerts untranslated');
            }
            else {
                logger_1.default.info(`Will restart the following services in order to sync UI dialogs translation: ` +
                    `${SERVICES_FOR_TRANSLATION}. This might have unexpected side effects, ` +
                    `see https://github.com/appium/appium/issues/19440 for more details`);
                await bluebird_1.default.all(SERVICES_FOR_TRANSLATION.map((arg) => this.simctl.spawnProcess([
                    'launchctl', 'stop', arg
                ])));
            }
        }
        return true;
    }
}
exports.default = SimulatorXcode9;
//# sourceMappingURL=simulator-xcode-9.js.map